# add by zhangmingze, 2014-2-10
from oslo_config import cfg
from oslo_utils import units
from oslo_log import log as logging
from cinder.i18n import _
from cinder import exception
from cinder.volume import driver
from cinder.volume.drivers.pcenter import client
LOG = logging.getLogger(__name__)

opt_group = cfg.OptGroup(name='pcenter',
                         title='Options for pcenter')
pcenter_opts = [
    cfg.StrOpt('api_url',
               default=None,
               help='IP address for connecting to pcenter server.'),
    cfg.StrOpt('username',
               default=None,
               help='Username for authenticating with pcenter server.'),
    cfg.StrOpt('password',
               default=None,
               help='Password for authenticating with pcenter server.'),
    cfg.StrOpt('version',
               default=None,
               help='Pcenter version'),
]

CONF = cfg.CONF
CONF.register_group(opt_group)
CONF.register_opts(pcenter_opts,group=opt_group)

#warning request timeout shoud be set in client ,we could cfg here
class PcenterDriver(driver.VolumeDriver):
    """Manage volumes on H3C flex server."""

    VERSION = ''

    def __init__(self, *args, **kwargs):
        super(PcenterDriver, self).__init__(*args, **kwargs)
        self.username = CONF.pcenter.username
        self.password = CONF.pcenter.password
        self.api_url = CONF.pcenter.api_url
        self.client = client.PcenterClient(self.api_url,
                                           self.username,
                                           self.password,
                                           None,
                                           None)

    def do_setup(self, context):
        """Perform validations and establish connection to server.
        :param context: Context information
        """
        pass
        
    def update_host_info(self,hostname):
        
        path = ('/host/%(hostname)s/resource' % {
            'hostname': hostname})
        response = self.client.get(path)
        status_code = response.status_code
        if status_code == 200 and response.json().get('code') == '2007' :
            return response.json()
        if status_code != 200 or response.json().get('code') != '0000': 
            msg = (_(
                'Error from pcenter while get host %(hostname)s info.'
                'Status code: %(status_code)d.'
                'error: %(error)s.'
                'Reason: %(reason)s ' % {'hostname': self.hostname,
                                          'status_code': status_code,
                                          'error': response.json().get('msg'),
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        return result
    def _get_hosts(self):
        path = ('/host')
        response = self.client.get(path)
        if response.status_code != 200 or response.json().get('code') != '0000':
            msg = (_(
                'Error from pcenter while get available nodes.'
                'Status code: %(status_code)d.'
                'error: %(error)s.'
                'Reason: %(reason)s ' % {'status_code': response.status_code,
                                         'error': response.json().get('msg'),
                                         'reason': response.reason}))
            raise exception.NovaException(msg)
        nodes = [ node for node in response.json().get('hosts')]
        return nodes
        
    def _get_stats(self):
        data = {'volume_backend_name': 'pcenter-driver',
                'vendor_name': 'pcenter',
                'driver_version': CONF.pcenter.version,
                'storage_protocol': 'LSI Logic SCSI',
                'reserved_percentage': 0,
                'total_capacity_gb': 0,
                'free_capacity_gb': 0}
        
        hosts = self._get_hosts()
        for hostname in hosts:
            info = self.update_host_info(hostname)
            data['total_capacity_gb'] += int (info['totalDisk']  * units.Mi / units.Gi)
            data['free_capacity_gb'] += int (info['usableDisk'] * units.Mi / units.Gi)
        return data

    def get_volume_stats(self, refresh=False):
        """Obtain status of the volume service.
        :param refresh: Whether to get refreshed information
        """
        LOG.error(_('begin to get volume stats'))
        if refresh:
            self.volume_stats = self._get_stats()
        return self.volume_stats
        LOG.error(_("success to get volume  state: %s") % self.volume_stats)
        

        
    def check_for_setup_error(self):
        pass

    def create_volume(self, volume):
        """Creates a new volume content:
        
        :param volume: Volume object
        """
        LOG.info(_('create a empty volume[%s] ,return success') % volume['name'])
         
    def ensure_export(self, context, volume):
        pass
    def create_export(self, context, volume):
        pass
    def remove_export(self, context, volume):
        pass
        

    def delete_volume(self, volume):
        """Deletes volume backing.

        :param volume: Volume object
        """
        LOG.info(_('delete  volume[%s] ,return success') % volume['name'])

    def create_snapshot(self, snapshot):
        """Creates a snapshot.
        :param snapshot: Snapshot object
        """
        raise NotImplementedError()

    def delete_snapshot(self, snapshot):
        """Delete snapshot.
        :param snapshot: Snapshot object
        """
        raise NotImplementedError()

    def _validate_disk_format(self,disk_format):
        """Verify vmdk as disk format.

        :param disk_format: Disk format of the image
        """
        pass

    def copy_image_to_volume(self, context, volume, image_service, image_id):
        """Creates volume from image.

        Creates a backing for the volume under the cas server and
        copies the image file from the glance image content.

        :param context: context
        :param volume: Volume object
        :param image_service: Glance image service
        :param image_id: Glance image id
        """
        raise NotImplementedError()

    def create_volume_from_snapshot(self, volume, snapshot):
        """Creates a volume from a snapshot.

        If the snapshot does not exist or source volume's backing does not
        exist, then pass.

        :param volume: New Volume object
        :param snapshot: Reference to snapshot entity
        """
        raise NotImplementedError()
        
    def initialize_connection(self, volume, connector):
        """Get information of volume's backing.
           If the volume does not have a backing yet. It will be created by CAS.

        :param volume: Volume object
        :param connector: Connector information
        :return: Return connection information
        """
        connection_info = {'driver_volume_type': "virtio"}
        connection_info['data'] = {'volumeName': volume['name'],
                                   'volumeSize':int(volume['size'] * units.Gi / units.Mi)}
        return connection_info
    
    def terminate_connection(self, volume, connector, force=False, **kwargs):
        pass       
